<?php

require_once 'Horde/DataTree.php';

/**
 * $Horde: trean/lib/Bookmarks.php,v 1.44 2005/01/06 19:02:08 chuck Exp $
 *
 * Copyright 2004-2005 Ben Chavet <ben@horde.org>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * @author  Ben Chavet <ben@horde.org>
 * @package Trean
 */
class Trean_Bookmarks {

    var $_datatree;

    /**
     * Constructor.  Sets some initial values.
     */
    function Trean_Bookmarks()
    {
        global $conf, $registry;

        if (empty($conf['datatree']['driver'])) {
            Horde::fatal('You must configure a DataTree backend to use Trean.', __FILE__, __LINE__);
        }

        $driver = $conf['datatree']['driver'];
        $this->_datatree = &Datatree::singleton(
            $driver,
            array_merge(Horde::getDriverConfig('datatree', $driver), array('group' => 'trean.bookmarks.' . Auth::getAuth()))
        );
    }

    /**
     * Return a new category object
     *
     * @param string $name       The category's datatree name
     * @param array $properties  The category's initial properties.
     *                           If set, a 'name' value is expected.
     *
     * @return object DataTreeObject_BookmarkCategory a new category
     *                                                object.
     */
    function &newCategory($name, $properties = null)
    {
        if (empty($name)) {
            return PEAR::raiseError(_("Category names must be non-empty"));
        }
        $category = &new DataTreeObject_BookmarkCategory($name, $properties);
        return $category;
    }

    /**
     * Return a new bookmark object
     *
     * @param string $name       The bookmark's datatree name.
     * @param array $properties  The bookmark's initial properties.
     *                           If set, 'url', 'title', and
     *                           'description' values are expected.
     *
     * @return object DataTreeObject_Bookmark  A new bookmark object.
     */
    function &newBookmark($name, $properties = null)
    {
        if (empty($name)) {
            return PEAR::raiseError(_("Bookmark names must be non-empty"));
        }
        $bookmark = &new DataTreeObject_Bookmark($name, $properties);
        return $bookmark;
    }

    /**
     * Return a DataTreeObject_BookmarkCategory object corresponding
     * to the given ID.
     *
     * @param int $id  The ID of the category to retrieve.
     *
     * @return  The category object corresponding to the given ID.
     */
    function &getCategory($id)
    {
        static $categoryCache = array();

        if (!isset($categoryCache[$id])) {
            $categoryCache[$id] = $this->_datatree->getObjectById($id, 'DataTreeObject_BookmarkCategory');
        }

        return $categoryCache[$id];
    }

    /**
     * List categories.
     *
     * @param integer $from     The bookmark to start fetching
     * @param integer $count     The numer of bookmarks to return
     */
    function &listCategories($parent = DATATREE_ROOT, $allLevels = true, $from = 0, $count = 0)
    {
        $criteria = array('AND' => array(
                        array('field' => 'name',
                              'op' => '=',
                              'test' => 'type'),
                        array('field' => 'value',
                              'op' => '=',
                              'test' => 'category')));
        $ids = &$this->_datatree->getByAttributes($criteria, $parent, $allLevels, true, $from, $count);
        if (is_a($ids, 'PEAR_Error')) {
            return $ids;
        }

        return $this->_datatree->getObjects(array_keys($ids), 'DataTreeObject_BookmarkCategory');
    }

    /**
     * Return a DataTreeObject_Bookmark object corresponding to the
     * given ID.
     *
     * @param int $id  The ID of the bookmark to retrieve.
     *
     * @return  The bookmark object corresponding to the given ID.
     */
    function &getBookmark($id)
    {
        static $bookmarkCache = array();

        if (!isset($bookmarkCache[$id])) {
            $bookmarkCache[$id] = $this->_datatree->getObjectById($id, 'DataTreeObject_Bookmark');
        }

        return $bookmarkCache[$id];
    }

    /**
     * List the bookmarks in this category.
     *
     * @param integer $from     The bookmark to start fetching
     * @param integer $count     The numer of bookmarks to return
     */
    function &listBookmarks($parent = DATATREE_ROOT, $from = 0, $count = 0)
    {
        $criteria = array('AND' => array(
                        array('field' => 'name',
                              'op' => '=',
                              'test' => 'type'),
                        array('field' => 'value',
                              'op' => '=',
                              'test' => 'bookmark')));
        $ids = &$this->_datatree->getByAttributes($criteria, $parent, false, true, $from, $count);
        if (is_a($ids, 'PEAR_Error')) {
            return $ids;
        }

        return $this->_datatree->getObjects(array_keys($ids), 'DataTreeObject_Bookmark');
    }

    /**
     * Adds a DataTreeObject_BookmarkCategory to the datatree backend.
     *
     * @param DataTreeObject_BookmarkCategory $category  The category to add.
     */
    function addCategory($category)
    {
        if (!is_a($category, 'DataTreeObject_BookmarkCategory')) {
            return PEAR::raiseError('Categories must be DataTreeObject_BookmarkCategory objects or extend that class.');
        }
        return $this->_datatree->add($category);
    }

    /**
     * Adds a DataTreeObject_Bookmark to the datatree backend.
     *
     * @param DataTreeObject_Bookmark $bookmark  The bookmark to add.
     */
    function addBookmark($bookmark)
    {
        if (!is_a($bookmark, 'DataTreeObject_Bookmark')) {
            return PEAR::raiseError('Bookmarks must be DataTreeObject_Bookmark objects or extend that class.');
        }
        return $this->_datatree->add($bookmark);
    }

    /**
     * Removes a DataTreeObject_BookmarkCategory from the datatree
     * backend.
     *
     * @param DataTreeObject_BookmarkCategory $category  The category to remove.
     * @param optional boolean $force [default = false]  Force the removal of every child.
     */
    function removeCategory($category, $force = false)
    {
        if (!is_a($category, 'DataTreeObject_BookmarkCategory')) {
            return PEAR::raiseError('Categories must be DataTreeObject_BookmarkCategory objects or extend that class.');
        }
        return $this->_datatree->remove($category, $force);
    }

    /**
     * Removes a DataTreeObject_Bookmark from the datatree backend.
     *
     * @param DataTreeObject_Bookmark  The bookmark to remove.
     * @param optional boolean $force [default = false] Force to
     *                         remove every child.
     */
    function removeBookmark($bookmark, $force = false)
    {
        if (!is_a($bookmark, 'DataTreeObject_Bookmark')) {
            return PEAR::raiseError('Bookmarks must be DataTreeObject_Bookmark objects or extend that class.');
        }
        return $this->_datatree->remove($bookmark, $force);
    }

    /**
     * Returns a list of bookmarks matching the specified criteria.
     *
     * @param array $criteria  Criteria to search with
     */
    function searchAllBookmarks($criteria)
    {
        return $this->_datatree->getByAttributes($criteria, DATATREE_ROOT, true, false);
    }

    /**
     * Returns the datatree name for the given ID
     */
    function getName($id)
    {
        return $this->_datatree->getName($id);
    }

    function getId($object)
    {
        return $this->_datatree->getId($object);
    }

    /**
     * Returns the datatree parent of the object with the given ID
     */
    function getParent($id)
    {
        return $this->_datatree->getParent($id);
    }

    function extractOneLevel($parent)
    {
        return $this->_datatree->_extractOneLevel($parent);
    }

}

/**
 * Extension of the DataTreeObject class for storing Bookmark
 * Categories in the datatree structure
 *
 * @author  Ben Chavet <ben@horde.org>
 * @package Trean
 */
class DataTreeObject_BookmarkCategory extends DataTreeObject {

    /**
     * Constructor.  Calls parent constructor and sets initial values.
     *
     * @param string $name       The category's datatree name.
     * @param array $properties  The category's initial properties.
     *                           If set, a 'name' value is expected.
     */
    function DataTreeObject_BookmarkCategory($name, $properties = null)
    {
        parent::DataTreeObject($name);

        $this->data['type'] = 'category';
        $this->data['name'] = isset($properties['name']) ? $properties['name'] : '';
    }

    /**
     * Set one of the attributes of the object.
     *
     * @param string   The attribute to set.
     * @param mixed    The value for $attribute.
     * @param boolean  Determines whether the backend should
     *                 be updated or not
     */
    function set($attribute, $value, $update = false)
    {
        parent::set($attribute, $value);
        if ($update) {
            $this->update();
        }
    }

    /**
     * Add a bookmark to this category
     *
     * @param array $properties  The initial properties for the new
     *                           bookmark.  Expected values are 'url',
     *                           'title', and 'description'.
     *
     * @return  The datatree ID for the new bookmark.
     */
    function addBookmark($properties)
    {
        global $trean_bookmarks;
        $bookmark = &$trean_bookmarks->newBookmark($this->getName() . ':' . md5(uniqid(mt_rand())), $properties);
        $trean_bookmarks->addBookmark($bookmark);
        return $trean_bookmarks->getId($bookmark);
    }

    /**
     * Add a child category to this category
     *
     * @param array $properties  The initial properties for the new
     *                           category.  Expected value is 'name'.
     *
     * @return  The datatree ID for the new category.
     */
    function addCategory($properties)
    {
        global $trean_bookmarks;
        $category = &$trean_bookmarks->newCategory($this->getName() . ':' . md5(uniqid(mt_rand())), $properties);
        $trean_bookmarks->addCategory($category);
        return $trean_bookmarks->getId($category);
    }

    /**      
     * Returns the datatree ID for this object. 
     */
    function getId() 
    {
        return $GLOBALS['trean_bookmarks']->_datatree->getId($this); 
    }

    /**     
     * Returns the datatree parent of this object. 
     */
    function getParent()   
    {
        return $GLOBALS['trean_bookmarks']->_datatree->getParent($this->getName());     
    }

    /**
     * Updates the datatree backend with the current values of this
     * category.
     */
    function update()
    {
        return $GLOBALS['trean_bookmarks']->_datatree->updateData($this);
    }

    /**
     * Deletes this category from the datatree backend.
     */
    function delete()
    {
        $GLOBALS['trean_bookmarks']->removeCategory($this, true);
    }

    function &listCategories($allLevels = true, $from = 0, $count = 0)
    {
        return $GLOBALS['trean_bookmarks']->listCategories($this->getName(), $allLevels, $from, $count);
    }

    function &listBookmarks($from = 0, $count = 0)
    {
        return $GLOBALS['trean_bookmarks']->listBookmarks($this->getName(), $from, $count);
    }

    /**
     * Map this object's attributes from the data array into a format
     * that we can store in the attributes storage backend.
     *
     * @return array  The attributes array.
     */
    function _toAttributes()
    {
        foreach ($this->data as $key => $value) {
            $attributes[] = array('name' => $key,
                                 'key' => '',
                                 'value' => $value);
        }
        return $attributes;
    }

    /**
     * Take in a list of attributes from the backend and map it to our
     * internal data array.
     *
     * @param array $attributes  The list of attributes from the
     *                           backend (attribute name, key,
     *                           and value).
     */
    function _fromAttributes($attributes)
    {
        foreach ($attributes as $attribute) {
            $this->data[$attribute['name']] = $attribute['value'];
        }
    }

}

/**
 * @author  Ben Chavet <ben@horde.org>
 * @package Trean
 */
class DataTreeObject_Bookmark extends DataTreeObject {

    function DataTreeObject_Bookmark($name, $properties = null)
    {
        parent::DataTreeObject($name);

        $this->data['type'] = 'bookmark';
        $this->data['url'] = isset($properties['url']) ? $properties['url'] : '';
        $this->data['title'] = isset($properties['title']) ? $properties['title'] : '';
        $this->data['description'] = isset($properties['description']) ? $properties['description'] : '';
    }

    /**
     * Set one of the attributes of the object.
     *
     * @param string   The attribute to set.
     * @param mixed    The value for $attribute.
     * @param boolean  Determines whether the backend should
     *                 be updated or not
     */
    function set($attribute, $value, $update = false)
    {
        parent::set($attribute, $value);
        if ($update) {
            $this->update();
        }
    }

    /**      
     * Returns the datatree ID for this object. 
     */
    function getId() 
    {
        return $GLOBALS['trean_bookmarks']->_datatree->getId($this); 
    }

    /**     
     * Returns the datatree parent of this object. 
     */
    function getParent()   
    {
        return $GLOBALS['trean_bookmarks']->_datatree->getParent($this->getName());     
    }

    /**
     * Updates the datatree backend with the current values of this
     * bookmark.
     */
    function update()
    {
        return $GLOBALS['trean_bookmarks']->_datatree->updateData($this);
    }

    /**
     * Deletes this bookmark from the datatree backend.
     */
    function delete()
    {
        return $GLOBALS['trean_bookmarks']->removeBookmark($this);
    }

    /**
     * Moves this bookmark to a new category.
     *
     * @param DataTreeObject_BookmarkCategory $newParent  The new parent of this bookmark.
     */
    function move($newParent)
    {
        return $GLOBALS['trean_bookmarks']->_datatree->move($this, $newParent);
    }

    /**
     * Map this object's attributes from the data array into a format
     * that we can store in the attributes storage backend.
     *
     * @return array  The attributes array.
     */
    function _toAttributes()
    {
        foreach ($this->data as $key => $value) {
            $attributes[] = array('name' => $key,
                                  'key' => '',
                                  'value' => $value);
        }
        return $attributes;
    }

    /**
     * Take in a list of attributes from the backend and map it to our
     * internal data array.
     *
     * @param array $attributes  The list of attributes from the
     *                           backend (attribute name, key,
     *                           and value).
     */
    function _fromAttributes($attributes)
    {
        foreach ($attributes as $attribute) {
            $this->data[$attribute['name']] = $attribute['value'];
        }
    }

}
